import { defineStore } from "pinia";
import type { User } from "@/client/auth";
import { UserService, RoleEnum } from "@/client/auth";
import { OpenAPI as S3ProxyOpenAPI } from "@/client/s3proxy";
import { OpenAPI as AuthOpenAPI } from "@/client/auth";
import { OpenAPI as WorkflowOpenAPI } from "@/client/workflow";
import { useWorkflowExecutionStore } from "@/stores/workflowExecutions";
import { useBucketStore } from "@/stores/buckets";
import { useWorkflowStore } from "@/stores/workflows";
import { useS3KeyStore } from "@/stores/s3keys";

type DecodedToken = {
  exp: number;
  iss: string;
  roles: RoleEnum[];
  sub: string;
};

export type RootState = {
  token: string | null;
  decodedToken: DecodedToken | null;
  user: User | null;
  userMapping: Record<string, string>;
};

function parseJwt(token: string): DecodedToken {
  const base64Url = token.split(".")[1];
  const base64 = base64Url.replace(/-/g, "+").replace(/_/g, "/");
  const jsonPayload = decodeURIComponent(
    window
      .atob(base64)
      .split("")
      .map(function (c) {
        return "%" + ("00" + c.charCodeAt(0).toString(16)).slice(-2);
      })
      .join(""),
  );

  return JSON.parse(jsonPayload) as DecodedToken;
}

export const useAuthStore = defineStore({
  id: "auth",
  state: () =>
    ({
      token: null,
      decodedToken: null,
      user: null,
      userMapping: {},
    }) as RootState,
  getters: {
    roles(): string[] {
      return (
        this.user?.roles?.map((role) => role.toString()) ??
        this.decodedToken?.["roles"] ??
        []
      );
    },
    authenticated: (state) => state.token != null,
    currentUID(): string {
      return this.decodedToken?.["sub"] ?? "";
    },
    foreignUser: (state) =>
      state.user?.roles?.includes(RoleEnum.FOREIGN_USER) ??
      state.decodedToken?.roles.includes(RoleEnum.FOREIGN_USER) ??
      true,
    normalUser: (state) =>
      state.user?.roles?.includes(RoleEnum.USER) ??
      state.decodedToken?.roles.includes(RoleEnum.USER) ??
      false,
    workflowReviewer: (state) =>
      state.user?.roles?.includes(RoleEnum.REVIEWER) ??
      state.decodedToken?.roles.includes(RoleEnum.REVIEWER) ??
      false,
    workflowDev: (state) =>
      state.user?.roles?.includes(RoleEnum.DEVELOPER) ??
      state.decodedToken?.roles.includes(RoleEnum.DEVELOPER) ??
      false,
    admin: (state) =>
      state.user?.roles?.includes(RoleEnum.ADMINISTRATOR) ??
      state.decodedToken?.roles.includes(RoleEnum.ADMINISTRATOR) ??
      false,
  },
  actions: {
    setToken(token: string | null) {
      if (token != null) {
        this.token = token;
        this.decodedToken = parseJwt(token);
        S3ProxyOpenAPI.TOKEN = token;
        AuthOpenAPI.TOKEN = token;
        WorkflowOpenAPI.TOKEN = token;
        UserService.userGetLoggedInUser()
          .then((user) => {
            this.updateUser(user);
          })
          .catch(() => {
            this.token = null;
          });
      } else {
        this.token = null;
        this.decodedToken = null;
        this.user = null;
      }
    },
    updateUser(user: User) {
      this.user = user;
      const keyRepository = useS3KeyStore();
      keyRepository.fetchS3Keys(user.uid);
    },
    logout() {
      this.$reset();
      useWorkflowExecutionStore().$reset();
      useBucketStore().$reset();
      useWorkflowStore().$reset();
      useS3KeyStore().$reset();
    },
    fetchUsernames(uids: string[]): Promise<User[]> {
      const filteredIds = uids
        .filter((uid) => !this.userMapping[uid]) // filter null modes and already present modes
        .filter(
          // filter unique workflow versions
          (modeId, index, array) =>
            array.findIndex((val) => val === modeId) === index,
        );
      return Promise.all(
        filteredIds.map((uid) => UserService.userGetUser(uid)),
      ).then((users) => {
        for (const user of users) {
          this.userMapping[user.uid] = user.display_name;
        }
        return users;
      });
    },
  },
});
